;; finder-site-lisp-generate.el -- generate ...

;; Copyright 2009, 2010, 2011, 2014 Kevin Ryde.




  ;; (eval-after-load "finder-inf" '(require 'finder-site-lisp-inf))





;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Keywords: maint
;; URL: http://user42.tuxfamily.org/finder-site-lisp-el/index.html

;; This file is part of finder-site-lisp-el.
;;
;; finder-site-lisp-el is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as published
;; by the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; finder-site-lisp-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with finder-site-lisp-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

;;; Commentary:

;; Usage: emacs -batch -no-site-file \
;;              -l finder-site-lisp-generate.el \
;; 	        -f finder-site-lisp-batch
;;
;;
;; FIXME: finder-compile-keywords uses insert-file-contents so errors out on
;; broken symlinks

;;; Code:

;;;###autoload (eval-after-load "finder" '(require 'finder-site-lisp-el))

(require 'advice)
(require 'finder)
(eval-when-compile (require 'cl))

;;-----------------------------------------------------------------------------
;; xemacs21 lacking

(eval-and-compile
  (defalias 'finder-site-lisp-make-temp-file
    (if (eval-when-compile (fboundp 'make-temp-file))
        'make-temp-file   ;; emacs
      ;; xemacs21
      (autoload 'mm-make-temp-file "mm-util") ;; from gnus
      'mm-make-temp-file)))

;;-----------------------------------------------------------------------------
;; generic

(defmacro finder-site-lisp-with-advice (function class name &rest body)
  "Evaluate BODY with advice on FUCNTION enabled.
Currently the advice is forcibly disabled afterwards, rather than
left at its original enabled or disabled."
  ;; checkdoc-params: (function class name &rest body)
  `(unwind-protect
       (progn
         (ad-enable-advice ',function ',class ',name)
         (ad-activate ',function)
         ,@body)
     (ad-disable-advice ',function ',class ',name)
     (ad-activate ',function)))
(put 'finder-site-lisp-with-advice 'lisp-indent-function 3)

(defmacro finder-site-lisp-with-tempdir (&rest body)
  "Evaluate BODY with a `tempdir' variable bound to a temporary directory.
The directory is deleted afterwards.  (Except currently not if
it's non-empty, though the intention would be to do a recursive
delete in that case.)"
  `(let ((tempdir (finder-site-lisp-make-temp-file "finder-site-lisp-" t)))
     (unwind-protect
         (let ((default-directory tempdir))
           ,@body)
       (condition-case nil (delete-directory tempdir) (error)))))

(defun finder-site-lisp-nonascii (str)
  "If STR is non-ascii then convert to a `decode-coding-string' lisp form.
If STR is all-ascii, or not a string, then it's returned
unchanged."

  (if (and (stringp str)
           (string-match "[^\000-\177]" str))
      (let ((coding buffer-file-coding-system)
            ret)
        (while str
          (cond ((string-match "\\`[\000-\177]+" str)
                 (push (match-string 0 str) ret)
                 (setq str (substring str (match-end 0))))
                ((string-match "\\`[^\000-\177]+" str)
                 (push `(condition-case nil
                            (decode-coding-string
                             (string ,@(string-to-list (encode-coding-string
                                                        (match-string 0 str)
                                                        coding)))
                             ',coding)
                          (error "?"))
                       ret)
                 (setq str (substring str (match-end 0))))
                (t
                 (setq str nil))))
        (cons 'unquote (cons 'concat (nreverse ret))))
    ;; all ascii, or not a string
    str))

;;-----------------------------------------------------------------------------

;; (defadvice lm-synopsis (after finder-site-lisp-el)
;;   "Apply `finder-site-lisp-nonascii' to the return."
;;   (setq ad-return-value (finder-site-lisp-nonascii ad-return-value)))
;; 
;; (defadvice lm-keywords-list (after finder-site-lisp-el)
;;   "Apply `finder-site-lisp-nonascii' to the return."
;;   (setq ad-return-value (mapcar 'finder-site-lisp-nonascii ad-return-value)))

(defun finder-site-lisp-generate ()
  "Generate finder-site-lisp-inf.el file in the current directory."

  (let ((origdir default-directory))
    (finder-site-lisp-with-tempdir

     (find-file "finder-inf.el")
     (setq buffer-file-coding-system 'iso-2022-7bit)
     (setq buffer-read-only nil)
     (erase-buffer)

     ;; look in the /site-lisp/ directories of `load-path'
     (let ((dirs (remove* "/site-lisp/" load-path
                          :test-not 'string-match)))

       ;; xemacs21 `finder-compile-keywords' tries to kill
       ;; "*finder-scratch*" even if it never created it
       (get-buffer-create "*finder-scratch*")

       ;; (finder-site-lisp-with-advice lm-synopsis after finder-site-lisp
       ;;   (finder-site-lisp-with-advice lm-keywords-list after finder-site-lisp
       (message "finder-compile-keywords ...")
       (apply 'finder-compile-keywords dirs))
     ;; ))
     (message "mangle to become finder-site-lisp-inf ...")

     (goto-char (point-min))
     (when (search-forward ";;; finder-inf.el" nil t)
       (replace-match ";;; finder-site-lisp-inf.el"))

     (goto-line 2)
     (if (looking-at ";;\n")
         (replace-match ""))
     (insert "
;; Generated by finder-site-lisp-generate.el -- DO NOT EDIT
;;
;; Copyright jointly to the authors of the respective packages,
;; at least to the extent that a one-line summary from each can
;; be copyright.

")

     (search-forward ";;; Code:\n")
     (insert "\n;;;###autoload (eval-after-load \"finder\" '(require 'finder-site-lisp-inf))\n")

     ;; xemacs21 oldish finder.el generates `defconst'
     (goto-char (point-min))
     (when (search-forward "(defconst finder-package-info" nil t)
       (replace-match "(setq finder-package-info" t t)
       (message "\
+-------------------------------------------------
| WARNING: Looks like old finder-compile-keywords
| from xemacs 21.4.  If that's so then the file
| /usr/share/emacs/site-lisp/finder-site-lisp-el.el
| will come out empty.
+-------------------------------------------------"))

     ;; `append' to builtin `finder-package-info'
     (goto-char (point-min))
     (re-search-forward "(setq finder-package-info")
     (replace-match "\
;; don't duplicate anything already in `finder-package-info'
 (let (extra)
   (mapcar (lambda (elem)
             (if (not (assoc (car elem) finder-package-info))
                 (setq extra (cons elem extra))))
")

     ;; (re-search-forward "'(")
     ;; (replace-match "`(")

     (while (re-search-forward "(unquote concat" nil t)
       (replace-match ",(concat"))

     (re-search-forward "))\n\f\\|^))\n")
     (goto-char (match-beginning 0))
     (insert "
   ))
   (setq finder-package-info (append finder-package-info (nreverse extra))")

     ;; don't suppress autoload generation, so can pick up cookie inserted
     ;; above (though it's not currently done that way)
     (goto-char (point-min))
     (when (re-search-forward "^;+ no-update-autoloads: t\n" nil t) ;; not xemacs21
       (replace-match ""))

     ;; xemacs21 doesn't have utf-8 coding builtin, drop to iso-2022-7bit
     (goto-char (point-min))
     (when (re-search-forward ";; coding: \\(.*\\)\n" nil t)
       (setq buffer-file-coding-system 'iso-2022-7bit)
       (message "decrease coding-system from %s to %s"
                (match-string 1) buffer-file-coding-system)
       (replace-match (format ";; coding: %s\n" buffer-file-coding-system)))

     (goto-char (point-min))
     (when (search-forward "(provide 'finder-inf)")
       (replace-match "(provide 'finder-site-lisp-inf)"))

     (goto-char (point-min))
     (message "saving ...")
     (save-buffer)

     (let ((target (expand-file-name "finder-site-lisp-inf.el" origdir)))
       (message "Move finder-inf.el to %s" target)
       (rename-file "finder-inf.el" target t)))))

(defun finder-site-lisp-generate-batch ()
  "Generate finder-site-lisp-inf.el in the current directory."
  (finder-site-lisp-generate)
  (kill-emacs 0))

;; (defun finder-site-lisp-directory ()
;;   (let ((case-fold-search t))
;;     (or (car (member* "[^0-9]/site-lisp\\'" (reverse load-path)
;;                       :test 'string-match))
;;         "/usr/share/emacs/site-lisp")))
;; 
;; (defun finder-site-lisp-generate-batch-install ()
;;   "Generate finder-site-lisp-inf.el and install to site-lisp."
;;   (let ((default-directory (finder-site-lisp-directory)))
;;     (finder-site-lisp-generate-batch)))

;; LocalWords: el symlinks ascii

;;; finder-site-lisp-generate.el ends here
